@@ -23,6 +23,8 @@ import * as Utils from '../utils.js';
 
 import {gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
+const FolderIconSize = 32;
+
 /**
  * Retrieves the folder's name, optionally translating it if the folder has a 'translate' flag.
  *
@@ -92,7 +94,7 @@ export class Layout extends BaseMenuLayout {
         this.topBox.add_child(this.searchEntry);
 
         // Applications Box - Contains Favorites, Categories or programs
-        this.applicationsScrollBox = this._createScrollBox({
+        this.applicationsScrollBox = this._createScrollView({
             x_expand: true,
             y_expand: true,
             y_align: Clutter.ActorAlign.START,
@@ -120,10 +122,10 @@ export class Layout extends BaseMenuLayout {
         this._setFolderGridColumns();
         this.foldersContainer.add_child(this.foldersGrid);
 
-        const homeGroupMenuItem = new HomeFolderMenuItem(this);
-        this._addItem(homeGroupMenuItem);
-        this.placeHolderFolderItem = new GroupFolderMenuItem(this, 'New Folder', null);
-        this._addItem(this.placeHolderFolderItem);
+        const homeFolderItem = new HomeFolderItem(this);
+        this._addItem(homeFolderItem);
+        const newFolderItem = new GroupFolderItem(this, 'New Folder', null);
+        this._addItem(newFolderItem);
 
         const searchBarLocation = ArcMenuManager.settings.get_enum('searchbar-default-top-location');
         if (searchBarLocation === Constants.SearchbarLocation.BOTTOM) {
@@ -205,8 +207,14 @@ export class Layout extends BaseMenuLayout {
         });
 
         this._orderedItems.forEach(icon => {
-            icon._redisplay();
+            icon.refreshContents();
         });
+
+        if (!this.activeCategoryItem || this.activeCategoryItem?.appIds.length === 0)
+            this.activateFolder();
+
+        this.fadeOutNewFolder();
+        this.maybeShowHomeFolder();
     }
 
     _addItem(item, position = 0) {
@@ -232,12 +240,6 @@ export class Layout extends BaseMenuLayout {
         this.foldersGrid.removeItem(item);
     }
 
-    activateHomeFolder() {
-        const homeFolder = this._folders.get('Library Home');
-        this.setActiveCategory(homeFolder);
-        homeFolder.displayAppList();
-    }
-
     updateWidth(setDefaultMenuView) {
         const widthAdjustment = ArcMenuManager.settings.get_int('menu-width-adjustment');
         let menuWidth = this.default_menu_width + widthAdjustment;
@@ -256,9 +258,8 @@ export class Layout extends BaseMenuLayout {
     _loadFolders() {
         const newFolders = [];
         const foldersData = {'Library Home': _('Library Home')};
-        const homeFolder = this._folders.get('Library Home');
-        newFolders.push(homeFolder);
-        const placeHolder = this._folders.get('New Folder');
+        const homeFolderItem = this._folders.get('Library Home');
+        newFolders.push(homeFolderItem);
 
         this._appInfoList = Shell.AppSystem.get_default().get_installed().filter(appInfo => {
             try {
@@ -275,13 +276,13 @@ export class Layout extends BaseMenuLayout {
 
         const appsInsideFolders = new Set();
 
-        // Load and Create GroupFolderMenuItems
+        // Load and Create GroupFolderItems
         const folders = this._folderSettings.get_strv('folder-children');
         folders.forEach(id => {
             const path = `${this._folderSettings.path}folders/${id}/`;
             let folderIcon = this._folders.get(id);
             if (!folderIcon) {
-                folderIcon = new GroupFolderMenuItem(this, id, path);
+                folderIcon = new GroupFolderItem(this, id, path);
                 folderIcon.connect('apps-changed', () => {
                     this._redisplay();
                 });
@@ -293,7 +294,7 @@ export class Layout extends BaseMenuLayout {
             if (!folderIcon.visible)
                 return;
 
-            folderIcon.getAppIds().forEach(appId => appsInsideFolders.add(appId));
+            folderIcon.appIds.forEach(appId => appsInsideFolders.add(appId));
             foldersData[id] = folderIcon.folder_name;
             newFolders.push(folderIcon);
         });
@@ -315,20 +316,28 @@ export class Layout extends BaseMenuLayout {
             const nameB = b.get_name();
             return nameA.localeCompare(nameB);
         });
-        homeFolder.appsList = remainingApps.map(app => app.id);
+        homeFolderItem.appIds = remainingApps.map(app => app.id);
 
-        placeHolder.set({
+        const homeFolderHasApps = homeFolderItem.appIds.length > 0;
+        homeFolderItem.set({
+            visible: homeFolderHasApps,
+            opacity: homeFolderHasApps ? 225 : 0,
+            scale_x: homeFolderHasApps ? 1 : 0,
+            scale_y: homeFolderHasApps ? 1 : 0,
+        });
+
+        const newFolderItem = this._folders.get('New Folder');
+        newFolderItem.set({
             visible: false,
             opacity: 0,
             scale_x: 0,
             scale_y: 0,
         });
-        newFolders.push(placeHolder);
+        newFolders.push(newFolderItem);
         return newFolders;
     }
 
     loadCategories() {
-
     }
 
     createNewFolder(app) {
@@ -362,9 +371,40 @@ export class Layout extends BaseMenuLayout {
         this._folderSettings.set_strv('folder-children', folders);
     }
 
-    fadeInPlaceHolder() {
-        this.placeHolderFolderItem.visible = true;
-        this.placeHolderFolderItem.ease({
+    maybeShowHomeFolder() {
+        const homeFolderItem = this._folders.get('Library Home');
+        const homeFolderHasApps = homeFolderItem.appIds.length > 0;
+
+        if (!homeFolderItem.visible && homeFolderHasApps) {
+            homeFolderItem.visible = true;
+            homeFolderItem.ease({
+                opacity: 255,
+                duration: 200,
+                scale_x: 1,
+                scale_y: 1,
+                mode: Clutter.AnimationMode.EASE_IN_QUAD,
+            });
+        } else if (!homeFolderHasApps) {
+            homeFolderItem.ease({
+                opacity: 0,
+                scale_x: 0,
+                scale_y: 0,
+                duration: 200,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                onComplete: () => {
+                    homeFolderItem.visible = false;
+                },
+            });
+        }
+    }
+
+    fadeInHomeFolder() {
+        const homeFolderItem = this._folders.get('Library Home');
+        if (homeFolderItem.visible)
+            return;
+
+        homeFolderItem.visible = true;
+        homeFolderItem.ease({
             opacity: 255,
             duration: 200,
             scale_x: 1,
@@ -373,15 +413,28 @@ export class Layout extends BaseMenuLayout {
         });
     }
 
-    fadeOutPlaceHolder() {
-        this.placeHolderFolderItem.ease({
+    fadeInNewFolder() {
+        const newFolderItem = this._folders.get('New Folder');
+        newFolderItem.visible = true;
+        newFolderItem.ease({
+            opacity: 255,
+            duration: 200,
+            scale_x: 1,
+            scale_y: 1,
+            mode: Clutter.AnimationMode.EASE_IN_QUAD,
+        });
+    }
+
+    fadeOutNewFolder() {
+        const newFolderItem = this._folders.get('New Folder');
+        newFolderItem.ease({
             opacity: 0,
             scale_x: 0,
             scale_y: 0,
             duration: 200,
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
             onComplete: () => {
-                this.placeHolderFolderItem.visible = false;
+                newFolderItem.visible = false;
             },
         });
     }
@@ -403,14 +456,29 @@ export class Layout extends BaseMenuLayout {
 
     setDefaultMenuView() {
         super.setDefaultMenuView();
+
+        this.activateFolder();
+    }
+
+    activateFolder() {
         const defaultView = ArcMenuManager.settings.get_string('pop-default-view');
-        let category = this._folders.get(defaultView);
+        let folder = this._folders.get(defaultView);
 
-        if (!category)
-            category = this._folders.values().next().value;
+        if (folder?.visible) {
+            folder.displayMenuItems();
+            this.setActiveCategory(folder);
+            return;
+        }
 
-        category.displayAppList();
-        this.setActiveCategory(category, true);
+        for (const folderIter of this._folders.values()) {
+            if (folderIter.visible) {
+                folder = folderIter;
+                break;
+            }
+        }
+
+        folder.displayMenuItems();
+        this.setActiveCategory(folder);
     }
 
     _onSearchEntryChanged(searchEntry, searchString) {
@@ -435,46 +503,44 @@ export class Layout extends BaseMenuLayout {
     }
 }
 
-class HomeFolderMenuItem extends MW.DraggableMenuItem {
+class BaseFolderItem extends MW.DraggableMenuItem {
     static [GObject.properties] = {
         'folder-name': GObject.ParamSpec.string('folder-name', 'folder-name', 'folder-name',
             GObject.ParamFlags.READWRITE, ''),
         'folder-id': GObject.ParamSpec.string('folder-id', 'folder-id', 'folder-id',
             GObject.ParamFlags.READWRITE, ''),
         'home-folder': GObject.ParamSpec.boolean('home-folder', 'home-folder', 'home-folder',
-            GObject.ParamFlags.READWRITE, true),
+            GObject.ParamFlags.READWRITE, false),
+        'new-folder': GObject.ParamSpec.boolean('new-folder', 'new-folder', 'new-folder',
+            GObject.ParamFlags.READWRITE, false),
     };
 
     static [GObject.signals] = {
         'apps-changed': {},
+        'folder-moved': {},
     };
 
     static {
         GObject.registerClass(this);
     }
 
-    constructor(menuLayout) {
+    constructor(menuLayout, isDraggable) {
         const displayType = Constants.DisplayType.GRID;
-        super(menuLayout, displayType, false);
+        super(menuLayout, displayType, isDraggable);
 
-        this.folder_id = 'Library Home';
-        this.folder_name = _('Library Home');
-
-        this._grid = this._menuLayout.applicationsGrid;
-        this.hasContextMenu = false;
-
-        this._items = new Map();
-        this._orderedItems = [];
+        this._grid = menuLayout.applicationsGrid;
+        this._menuItems = [];
+        this._menuItemsMap = new Map();
 
         this.add_style_class_name('ArcMenuIconGrid ArcMenuGroupFolder');
 
         this._iconBin = new St.Bin();
         this.add_child(this._iconBin);
-
-        this.label.text = _('Library Home');
         this.add_child(this.label);
         this._updateIcon();
 
+        this.bind_property('folder-name', this.label, 'text', GObject.BindingFlags.SYNC_CREATE);
+
         this.set({
             ...Utils.getOrientationProp(true),
             x_expand: false,
@@ -483,87 +549,62 @@ class HomeFolderMenuItem extends MW.DraggableMenuItem {
         });
     }
 
-    _isFolderDisplayed() {
+    get isActiveFolder() {
         return this._menuLayout.activeCategoryItem === this;
     }
 
+    _compareItems(a, b) {
+        return a.name.localeCompare(b.name);
+    }
+
+    getDragActor() {
+        return this.createIcon();
+    }
+
+    _canAccept(source) {
+        return source !== this && source instanceof ApplicationMenuItem && !this._menuItems.includes(source);
+    }
+
+    acceptDrop(source, actor, x, y) {
+        const acceptDrop = super.acceptDrop(source, actor, x, y);
+        if (!acceptDrop)
+            return false;
+
+        const {folderMenuItem, _app: app} = source;
+
+        if (!app || !folderMenuItem)
+            return false;
+
+        source.dragDropAccepted = true;
+
+        if (!folderMenuItem.homeFolder)
+            folderMenuItem.removeApp(app);
+
+        if (this.new_folder) {
+            this._menuLayout.createNewFolder(app);
+            return true;
+        }
+
+        if (!this.home_folder)
+            this.addApp(app);
+
+        return true;
+    }
+
     _showFolderPreview() {
     }
 
     _hideFolderPreview() {
     }
 
-    acceptDrop(source, actor, x, y) {
-        const acceptDrop = super.acceptDrop(source, actor, x, y);
-        if (!acceptDrop)
-            return false;
-
-        const app = source._app;
-        const {folderMenuItem} = source;
-
-        folderMenuItem?.removeApp(app);
-
-        this._menuLayout.fadeOutPlaceHolder();
-        return true;
-    }
-
-    _canAccept(source) {
-        return source !== this && source instanceof ApplicationMenuItem && !this._apps.includes(source._app);
-    }
-
-    createIcon() {
-        const iconSize = 32;
-
-        const icon = new St.Icon({
-            style_class: 'popup-menu-icon',
-            icon_size: iconSize,
-            icon_name: 'user-home-symbolic',
-        });
-        return icon;
-    }
-
-    getAppIds() {
-        return this._orderedItems.map(item => item._app.id);
-    }
-
-    displayAppList() {
-        this._menuLayout._clearActorsFromBox();
-        this._menuLayout.searchEntry?.clearWithoutSearchChangeEvent();
-        this._grid.removeAllItems();
-        this._menuLayout._setGridColumns(this._grid);
-        const groupAllAppsGridView = ArcMenuManager.settings.get_boolean('group-apps-alphabetically-grid-layouts');
-        let currentCharacter;
-
-        for (let i = 0; i < this._orderedItems.length; i++) {
-            const item = this._orderedItems[i];
-            const parent = item.get_parent();
-            if (parent)
-                parent.remove_child(item);
-
-            if (groupAllAppsGridView) {
-                const appNameFirstChar = item._app.get_name().charAt(0).toLowerCase();
-                if (currentCharacter !== appNameFirstChar) {
-                    currentCharacter = appNameFirstChar;
-
-                    const label = this._menuLayout._createLabelWithSeparator(currentCharacter.toUpperCase());
-                    this._grid.appendItem(label);
-                }
-            }
-
-            this._grid.appendItem(item);
-        }
-
-        if (this._menuLayout.applicationsBox && !this._menuLayout.applicationsBox.contains(this._grid))
-            this._menuLayout.applicationsBox.add_child(this._grid);
-
-        this._menuLayout.activeMenuItem = this._orderedItems[0];
-    }
-
     _loadApps() {
-        this._apps = [];
-
+        const apps = [];
+        const excludedApps = this._folder?.get_strv('excluded-apps') || [];
         const appSys = Shell.AppSystem.get_default();
         const addAppId = appId => {
+            if (excludedApps.includes(appId))
+                return;
+
             const app = appSys.lookup_app(appId);
             if (!app)
                 return;
@@ -571,17 +612,29 @@ class HomeFolderMenuItem extends MW.DraggableMenuItem {
             if (!ParentalControlsManager.getDefault().shouldShowApp(app.get_app_info()))
                 return;
 
-            if (this._apps.indexOf(app) !== -1)
+            if (apps.indexOf(app) !== -1)
                 return;
 
-            this._apps.push(app);
+            apps.push(app);
         };
 
-        this.appsList.forEach(addAppId);
+        this.appIds.forEach(addAppId);
+
+        if (this._folder) {
+            const folderCategories = this._folder.get_strv('categories');
+            const appInfos = this._menuLayout.getAppInfos();
+            appInfos.forEach(appInfo => {
+                const appCategories = Utils.getCategories(appInfo);
+                if (!listsIntersect(folderCategories, appCategories))
+                    return;
+
+                addAppId(appInfo.get_id());
+            });
+        }
 
         const items = [];
-        this._apps.forEach(app => {
-            let icon = this._items.get(app.get_id());
+        apps.forEach(app => {
+            let icon = this._menuItemsMap.get(app.get_id());
             if (!icon) {
                 icon = new ApplicationMenuItem(this._menuLayout, app, this._menuLayout.display_type);
                 icon.setFolderGroup(this);
@@ -593,12 +646,11 @@ class HomeFolderMenuItem extends MW.DraggableMenuItem {
         return items;
     }
 
-    _compareItems(a, b) {
-        return a.name.localeCompare(b.name);
-    }
+    refreshContents() {
+        if (this.new_folder)
+            return;
 
-    _redisplay() {
-        const oldApps = this._orderedItems.slice();
+        const oldApps = this._menuItems.slice();
         const oldAppIds = oldApps.map(icon => icon._app.id);
 
         const newApps = this._loadApps();
@@ -616,7 +668,7 @@ class HomeFolderMenuItem extends MW.DraggableMenuItem {
         // Add new app icons, or move existing ones
         newApps.forEach(item => {
             const index = newApps.indexOf(item);
-            const position = this._orderedItems.indexOf(item);
+            const position = this._menuItems.indexOf(item);
             if (addedApps.includes(item))
                 this._addItem(item, index);
             else if (position !== index)
@@ -624,114 +676,187 @@ class HomeFolderMenuItem extends MW.DraggableMenuItem {
         });
     }
 
-    _addItem(item, position) {
-        this._items.set(item._app.id, item);
+    displayMenuItems() {
+        this._menuLayout._clearActorsFromBox();
+        this._menuLayout.searchEntry?.clearWithoutSearchChangeEvent();
+        this._grid.removeAllItems();
+        this._menuLayout._setGridColumns(this._grid);
+        const groupAllAppsGridView = ArcMenuManager.settings.get_boolean('group-apps-alphabetically-grid-layouts');
+        let currentCharacter;
 
-        if (this._isFolderDisplayed())
+        for (let i = 0; i < this._menuItems.length; i++) {
+            const item = this._menuItems[i];
+            const parent = item.get_parent();
+            if (parent)
+                parent.remove_child(item);
+
+            if (groupAllAppsGridView && this.home_folder) {
+                const appNameFirstChar = item._app.get_name().charAt(0).toLowerCase();
+                if (currentCharacter !== appNameFirstChar) {
+                    currentCharacter = appNameFirstChar;
+
+                    const label = this._menuLayout._createLabelWithSeparator(currentCharacter.toUpperCase());
+                    this._grid.appendItem(label);
+                }
+            }
+
+            this._grid.appendItem(item);
+        }
+
+        if (this._menuLayout.applicationsBox && !this._menuLayout.applicationsBox.contains(this._grid))
+            this._menuLayout.applicationsBox.add_child(this._grid);
+
+        this._menuLayout.activeMenuItem = this._menuItems[0];
+    }
+
+    _addItem(item, position) {
+        this._menuItemsMap.set(item._app.id, item);
+
+        if (this.isActiveFolder)
             this._grid.addItem(item, position);
 
-        this._orderedItems.splice(position, 0, item);
+        this._menuItems.splice(position, 0, item);
     }
 
     _moveItem(item, newPosition) {
-        if (this._isFolderDisplayed())
+        if (this.isActiveFolder)
             this._grid.moveItem(item, newPosition);
 
-        // Update the _orderedItems array
-        this._orderedItems.splice(this._orderedItems.indexOf(item), 1);
-        this._orderedItems.splice(newPosition, 0, item);
+        const oldIndex = this._menuItems.indexOf(item);
+        this._menuItems.splice(oldIndex, 1);
+        this._menuItems.splice(newPosition, 0, item);
     }
 
     _removeItem(item) {
-        const iconIndex = this._orderedItems.indexOf(item);
+        const oldIndex = this._menuItems.indexOf(item);
+        this._menuItems.splice(oldIndex, 1);
+        this._menuItemsMap.delete(item._app.id);
 
-        this._orderedItems.splice(iconIndex, 1);
-        this._items.delete(item._app.id);
-        if (this._isFolderDisplayed())
+        if (this.isActiveFolder)
             this._grid.removeItem(item);
     }
 
     activate(event) {
         super.activate(event);
         this._menuLayout.setActiveCategory(this);
-        this.displayAppList();
+        this.displayMenuItems();
+    }
+
+    _onDestroy() {
+        if (this._menuLayout.activeCategoryItem === this)
+            this._menuLayout.setActiveCategory(null, false);
+
+        super._onDestroy();
     }
 }
 
-class GroupFolderMenuItem extends MW.DraggableMenuItem {
-    static [GObject.properties] = {
-        'folder-name': GObject.ParamSpec.string('folder-name', 'folder-name', 'folder-name',
-            GObject.ParamFlags.READWRITE, ''),
-        'folder-id': GObject.ParamSpec.string('folder-id', 'folder-id', 'folder-id',
-            GObject.ParamFlags.READWRITE, ''),
-        'new-folder': GObject.ParamSpec.boolean('new-folder', 'new-folder', 'new-folder',
-            GObject.ParamFlags.READWRITE, false),
-    };
+class HomeFolderItem extends BaseFolderItem {
+    static {
+        GObject.registerClass(this);
+    }
 
-    static [GObject.signals] = {
-        'folder-moved': {},
-        'apps-changed': {},
-    };
+    constructor(menuLayout) {
+        super(menuLayout, false);
 
+        this.hasContextMenu = false;
+        this.folder_id = 'Library Home';
+        this.folder_name = _('Library Home');
+        this.home_folder = true;
+        this._appIds = [];
+    }
+
+    get appIds() {
+        return this._appIds;
+    }
+
+    set appIds(appIds) {
+        this._appIds = appIds;
+    }
+
+    createIcon() {
+        const icon = new St.Icon({
+            style_class: 'popup-menu-icon',
+            icon_size: FolderIconSize,
+            icon_name: 'user-home-symbolic',
+        });
+        return icon;
+    }
+}
+
+class GroupFolderItem extends BaseFolderItem {
     static {
         GObject.registerClass(this);
     }
 
     constructor(menuLayout, id, path) {
-        const displayType = Constants.DisplayType.GRID;
-        super(menuLayout, displayType);
+        super(menuLayout, true);
 
         if (path) {
             this._folder = new Gio.Settings({
                 schema_id: 'org.gnome.desktop.app-folders.folder',
                 path,
             });
+            this._folder.connectObject('changed', this._sync.bind(this), this);
         } else {
             this.new_folder = true;
         }
-        this._apps = [];
-        this._grid = this._menuLayout.applicationsGrid;
+
+        this.hasContextMenu = true;
         this.folder_id = id;
         this.hasContextMenu = true;
-        this._items = new Map();
-        this._orderedItems = [];
 
-        this.add_style_class_name('ArcMenuIconGrid ArcMenuGroupFolder');
-        this.set({
-            ...Utils.getOrientationProp(true),
-            x_expand: false,
-            tooltipLocation: Constants.TooltipLocation.BOTTOM_CENTERED,
-            style: `width: ${110}px; height: ${72}px;`,
-        });
-
-        this.add_child(this.label);
-
-
-        if (this._folder) {
-            this._folder.connectObject(
-                'changed', this._sync.bind(this), this);
-        }
-        this._redisplay();
+        this.refreshContents();
         this._sync();
     }
 
-    _isFolderDisplayed() {
-        return this._menuLayout.activeCategoryItem === this;
+    get appIds() {
+        return this._folder?.get_strv('apps') || [];
+    }
+
+    createIcon() {
+        if (!this._menuItems.length || this.new_folder) {
+            const icon = new St.Icon({
+                style_class: 'popup-menu-icon',
+                icon_size: FolderIconSize,
+                icon_name: 'folder-directory-symbolic',
+            });
+            return icon;
+        }
+
+        const layout = new Clutter.GridLayout({
+            row_homogeneous: true,
+            column_homogeneous: true,
+        });
+        const icon = new St.Widget({
+            layout_manager: layout,
+            x_align: Clutter.ActorAlign.CENTER,
+            style: `width: ${FolderIconSize}px; height: ${FolderIconSize}px;`,
+        });
+
+        const subSize = Math.floor(.4 * FolderIconSize);
+
+        const numItems = this._menuItems.length;
+        const rtl = icon.get_text_direction() === Clutter.TextDirection.RTL;
+        for (let i = 0; i < 4; i++) {
+            const style = `width: ${subSize}px; height: ${subSize}px;`;
+            const bin = new St.Bin({style});
+            if (i < numItems)
+                bin.child = this._menuItems[i]._app.create_icon_texture(subSize);
+            layout.attach(bin, rtl ? (i + 1) % 2 : i % 2, Math.floor(i / 2), 1, 1);
+        }
+
+        return icon;
     }
 
     _updateName() {
         if (this.new_folder) {
             this.folder_name = _('New Folder');
-            this.label.text = this.folder_name;
             return;
         }
 
         const name = getFolderName(this._folder);
-        if (this.folder_name === name)
-            return;
-
-        this.folder_name = name;
-        this.label.text = this.folder_name;
+        if (this.folder_name !== name)
+            this.folder_name = name;
     }
 
     _sync() {
@@ -739,9 +864,11 @@ class GroupFolderMenuItem extends MW.DraggableMenuItem {
             return;
 
         this.emit('apps-changed');
+
         this._updateName();
+
         if (!this.new_folder)
-            this.visible = this._apps.length > 0;
+            this.visible = this._menuItems.length > 0;
 
         this._updateIcon();
     }
@@ -772,9 +899,6 @@ class GroupFolderMenuItem extends MW.DraggableMenuItem {
         folders.splice(folders.indexOf(this.folder_id), 1);
         settings.set_strv('folder-children', folders);
 
-        // if the folder is now deleted, activate the library home folder
-        this._menuLayout.activateHomeFolder();
-
         this._deletingFolder = false;
     }
 
@@ -784,8 +908,7 @@ class GroupFolderMenuItem extends MW.DraggableMenuItem {
         if (index >= 0)
             folderApps.splice(index, 1);
 
-        // Remove the folder if this is the last app icon; otherwise,
-        // just remove the icon
+        // Remove the folder if this is the last app icon; otherwise, just remove the icon
         if (folderApps.length === 0) {
             this._removeFolder();
         } else {
@@ -803,11 +926,6 @@ class GroupFolderMenuItem extends MW.DraggableMenuItem {
     }
 
     popupContextMenu() {
-        if (this.home_folder)
-            return;
-        if (this.tooltip)
-            this.tooltip.hide();
-
         if (this.contextMenu === undefined) {
             this.contextMenu = new PopupMenu.PopupMenu(this, 0.5, St.Side.TOP);
             this.contextMenu.connect('open-state-changed', (menu, isOpen) => {
@@ -909,43 +1027,6 @@ class GroupFolderMenuItem extends MW.DraggableMenuItem {
         dialog.open();
     }
 
-    getDragActor() {
-        return this.createIcon();
-    }
-
-    _showFolderPreview() {
-    }
-
-    _hideFolderPreview() {
-    }
-
-    acceptDrop(source, actor, x, y) {
-        const acceptDrop = super.acceptDrop(source, actor, x, y);
-        if (!acceptDrop)
-            return false;
-
-        const app = source._app;
-        const {folderMenuItem} = source;
-
-        if (!folderMenuItem.home_folder)
-            folderMenuItem?.removeApp(app);
-
-        if (this.new_folder) {
-            this._menuLayout.createNewFolder(app);
-            return true;
-        }
-
-        this.addApp(app);
-
-        this._menuLayout.fadeOutPlaceHolder();
-
-        return true;
-    }
-
-    _canAccept(source) {
-        return source !== this && source instanceof ApplicationMenuItem && !this._apps.includes(source._app);
-    }
-
     _getDropTarget(x, y, source, layoutManager) {
         const [targetIndex, dragLocation] = super._getDropTarget(x, y, source, layoutManager);
 
@@ -971,188 +1052,6 @@ class GroupFolderMenuItem extends MW.DraggableMenuItem {
         });
         this._menuLayout._folderSettings.set_strv('folder-children', orderedFolders);
     }
-
-    createIcon() {
-        const iconSize = 32;
-
-        if (!this._apps.length || this.new_folder) {
-            const icon = new St.Icon({
-                style_class: 'popup-menu-icon',
-                icon_size: iconSize,
-                icon_name: 'folder-directory-symbolic',
-            });
-            return icon;
-        }
-
-        const layout = new Clutter.GridLayout({
-            row_homogeneous: true,
-            column_homogeneous: true,
-        });
-        const icon = new St.Widget({
-            layout_manager: layout,
-            x_align: Clutter.ActorAlign.CENTER,
-            style: `width: ${iconSize}px; height: ${iconSize}px;`,
-        });
-
-        const subSize = Math.floor(.4 * iconSize);
-
-        const numItems = this._apps.length;
-        const rtl = icon.get_text_direction() === Clutter.TextDirection.RTL;
-        for (let i = 0; i < 4; i++) {
-            const style = `width: ${subSize}px; height: ${subSize}px;`;
-            const bin = new St.Bin({style});
-            if (i < numItems)
-                bin.child = this._apps[i].create_icon_texture(subSize);
-            layout.attach(bin, rtl ? (i + 1) % 2 : i % 2, Math.floor(i / 2), 1, 1);
-        }
-
-        return icon;
-    }
-
-    getAppIds() {
-        return this._folder.get_strv('apps');
-    }
-
-    displayAppList() {
-        this._menuLayout._clearActorsFromBox();
-        this._menuLayout.searchEntry?.clearWithoutSearchChangeEvent();
-
-        this._grid.removeAllItems();
-
-        this._menuLayout._setGridColumns(this._grid);
-
-        for (let i = 0; i < this._orderedItems.length; i++) {
-            const item = this._orderedItems[i];
-            const parent = item.get_parent();
-            if (parent)
-                parent.remove_child(item);
-
-            this._grid.appendItem(item);
-        }
-
-        if (this._menuLayout.applicationsBox && !this._menuLayout.applicationsBox.contains(this._grid))
-            this._menuLayout.applicationsBox.add_child(this._grid);
-
-        this._menuLayout.activeMenuItem = this._orderedItems[0];
-    }
-
-    _loadApps() {
-        this._apps = [];
-        const excludedApps = this._folder.get_strv('excluded-apps');
-        const appSys = Shell.AppSystem.get_default();
-        const addAppId = appId => {
-            if (excludedApps.includes(appId))
-                return;
-
-            const app = appSys.lookup_app(appId);
-            if (!app)
-                return;
-
-            if (!ParentalControlsManager.getDefault().shouldShowApp(app.get_app_info()))
-                return;
-
-            if (this._apps.indexOf(app) !== -1)
-                return;
-
-            this._apps.push(app);
-        };
-
-        const folderApps = this._folder.get_strv('apps');
-        folderApps.forEach(addAppId);
-
-        const folderCategories = this._folder.get_strv('categories');
-        const appInfos = this._menuLayout.getAppInfos();
-        appInfos.forEach(appInfo => {
-            const appCategories = Utils.getCategories(appInfo);
-            if (!listsIntersect(folderCategories, appCategories))
-                return;
-
-            addAppId(appInfo.get_id());
-        });
-
-        const items = [];
-        this._apps.forEach(app => {
-            let icon = this._items.get(app.get_id());
-            if (!icon) {
-                icon = new ApplicationMenuItem(this._menuLayout, app, this._menuLayout.display_type);
-                icon.setFolderGroup(this);
-            }
-
-            items.push(icon);
-        });
-
-        return items;
-    }
-
-    _compareItems(a, b) {
-        return a.name.localeCompare(b.name);
-    }
-
-    _redisplay() {
-        if (this.new_folder)
-            return;
-        const oldApps = this._orderedItems.slice();
-        const oldAppIds = oldApps.map(icon => icon._app.id);
-
-        const newApps = this._loadApps();
-        const newAppIds = newApps.map(icon => icon._app.id);
-
-        const addedApps = newApps.filter(icon => !oldAppIds.includes(icon._app.id));
-        const removedApps = oldApps.filter(icon => !newAppIds.includes(icon._app.id));
-
-        // Remove old app icons
-        removedApps.forEach(item => {
-            this._removeItem(item);
-            item.destroy();
-        });
-
-        // Add new app icons, or move existing ones
-        newApps.forEach(item => {
-            const index = newApps.indexOf(item);
-            const position = this._orderedItems.indexOf(item);
-            if (addedApps.includes(item)) {
-                this._addItem(item, index);
-            } else if (position !== index) {
-                this._moveItem(item, index);
-            } else {
-                this._orderedItems.splice(this._orderedItems.indexOf(item), 1);
-                this._orderedItems.splice(index, 0, item);
-            }
-        });
-    }
-
-    _addItem(item, position) {
-        this._items.set(item._app.id, item);
-
-        if (this._isFolderDisplayed())
-            this._grid.addItem(item, position);
-
-        this._orderedItems.splice(position, 0, item);
-    }
-
-    _moveItem(item, newPosition) {
-        if (this._isFolderDisplayed())
-            this._grid.moveItem(item, newPosition);
-
-        // Update the _orderedItems array
-        this._orderedItems.splice(this._orderedItems.indexOf(item), 1);
-        this._orderedItems.splice(newPosition, 0, item);
-    }
-
-    _removeItem(item) {
-        const iconIndex = this._orderedItems.indexOf(item);
-
-        this._orderedItems.splice(iconIndex, 1);
-        this._items.delete(item._app.id);
-        if (this._isFolderDisplayed())
-            this._grid.removeItem(item);
-    }
-
-    activate(event) {
-        super.activate(event);
-        this._menuLayout.setActiveCategory(this);
-        this.displayAppList();
-    }
 }
 
 export class ApplicationMenuItem extends MW.DraggableMenuItem {
@@ -1167,8 +1066,8 @@ export class ApplicationMenuItem extends MW.DraggableMenuItem {
 
         this.hasContextMenu = !!this._app;
 
-        const disableRecentAppsIndicator = ArcMenuManager.settings.get_boolean('disable-recently-installed-apps');
-        if (!disableRecentAppsIndicator) {
+        const showNewApps = ArcMenuManager.settings.get_boolean('show-recently-installed-apps');
+        if (showNewApps) {
             const recentApps = ArcMenuManager.settings.get_strv('recently-installed-apps');
             this.isRecentlyInstalled = recentApps.some(appIter => appIter === this._app.get_id());
         }
@@ -1215,7 +1114,8 @@ export class ApplicationMenuItem extends MW.DraggableMenuItem {
     }
 
     _onDragBegin() {
-        this._menuLayout.fadeInPlaceHolder();
+        this._menuLayout.fadeInNewFolder();
+        this._menuLayout.fadeInHomeFolder();
         super._onDragBegin();
     }
 
@@ -1235,11 +1135,16 @@ export class ApplicationMenuItem extends MW.DraggableMenuItem {
     }
 
     _onDragEnd() {
-        this._menuLayout.fadeOutPlaceHolder();
+        this._menuLayout?.fadeOutNewFolder();
+        if (!this.dragDropAccepted)
+            this._menuLayout?.maybeShowHomeFolder();
         super._onDragEnd();
 
+        if (!this._menuLayout || this.dragDropAccepted)
+            return;
+
         const parent = this.get_parent();
-        if (!parent || (this.folderMenuItem && this.folderMenuItem.home_folder))
+        if (!parent || this.folderMenuItem?.home_folder)
             return;
 
         const layoutManager = parent.layout_manager;
@@ -1292,8 +1197,6 @@ export class ApplicationMenuItem extends MW.DraggableMenuItem {
 
     popupContextMenu() {
         this.removeIndicator();
-        if (this.tooltip)
-            this.tooltip.hide();
 
         if (!this._app && !this.folderPath)
             return;
